/**
 * Copyright (c) 2018-present, Umweltbundesamt GmbH
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import React from 'react';
import { IValidationState } from '../withValidation';
import { IValidationSummaryProps } from './ValidationSummary.types';
interface IValidationEventArgs extends IValidationState {
    label: string;
}
interface IFieldContainer {
    [s: string]: IValidationEventArgs;
}
interface IValidationSummaryState {
    fields: IFieldContainer;
}
/**
 * Component for displaying a summary of all
 * validation errors of the form this component
 * lives in.
 */
export declare class BaseValidationSummary extends React.Component<IValidationSummaryProps, IValidationSummaryState> {
    static displayName: string;
    static defaultProps: {
        title: string;
        disableFocusOnSubmit: boolean;
    };
    private headerRef;
    constructor(props: IValidationSummaryProps);
    /**
     * Renders a field error
     * @param id Unique id
     * @param fieldName Name of the field
     * @param errors Field errors
     * @param linkCallback Callback for the error click
     */
    private static renderError;
    /**
     * Unregisters the field from the form
     */
    componentWillUnmount(): void;
    /**
     * Gets called when a validation state changes
     * @param name Field name
     * @param event Event name
     * @param state Field state
     */
    private notify;
    private scrollIntoView;
    /**
     * Renders a list of all errors
     * @param errors Error array
     */
    private renderErrors;
    render(): JSX.Element | null;
}
export declare const ValidationSummary: React.ComponentType<Pick<Pick<IValidationSummaryProps, "id" | "renderFieldError" | "render" | "context"> & Partial<Pick<IValidationSummaryProps, "title" | "disableFocusOnSubmit">> & Partial<Pick<{
    title: string;
    disableFocusOnSubmit: boolean;
}, never>>, "id" | "title" | "disableFocusOnSubmit" | "renderFieldError" | "render">>;
export {};
